#include "cvManager.h"

cvManager::cvManager() {
}

//--------------------------------------------------------------
void cvManager::setup() {

    w = 640;
    h = 480;
    cvInputColor.allocate(w, h);
    cvInputGray.allocate(w,h);
    blobAsTempate = new ofxCvGrayscaleImage();
    blobAsTempate->allocate(64,64);

    drawMode = 0;

    loadSettings();

    minArea = 180; // save this out
    maxArea = 600;
    minRatio = 1.2f;

    snapshotCounter = 0;

    NL.loadNumbers();

}


//--------------------------------------------------------------
void cvManager::update(ofxCvColorImage & pixels) {




    cvInputColor = pixels;

    // ------------------------------------------------------------------------- go from color to grayscale ( and deinterlace )
    cvInputGray.setFromColorImage(cvInputColor);
    CS.findCardContents(cvInputGray);

    // ------------------------------------------------------------------------- find the contours of the numbers:
    ofCvF.findContours(CS.cvInputDiff, 20,1000,128,false);

    for (int i = 0; i < 128; i++) {
        numbers[i].reset();
    }

    // -------------------------------------------------------------------- are you a number?
    nNumbers = ofCvF.nBlobs;
    for (int i = 0; i < ofCvF.nBlobs; i++) {
        numbers[i].setFromBlobAndImage(ofCvF.blobs[i], cvInputGray);
        if ( ofCvF.blobs[i].boxArea > minArea && ofCvF.blobs[i].boxArea < maxArea
                && ofCvF.blobs[i].ratio > minRatio) {
            numbers[i].bPossibleNumber = true;
        } else {
            numbers[i].bPossibleNumber = false;
        }
    }


    // maybe this should be done elsewhere?

    ofPoint dst[4];
    dst[0].x = 0;
    dst[0].y = 0;
    dst[1].x = 64;
    dst[1].y = 0;
    dst[2].x = 64;
    dst[2].y = 64;
    dst[3].x = 0;
    dst[3].y = 64;

    for (int i = 0; i < ofCvF.nBlobs; i++) {
        if (numbers[i].bPossibleNumber == true) {

            blobAsTempate->warpIntoMe(cvInputGray, numbers[i].quadPts, dst);
            blobAsTempate->contrastStretch();

            float minimumDiff = 10000000;
            int   minNum = -1;
            float angleOfMin = PI/2.0f;

            for (int j = 0; j < 13; j++) {
                int nImages = NL.numbers[j].images.size();
                for (int k = 0; k < nImages; k++) {
                    float diff = NC.totalDifference( NL.numbers[j].images[k], blobAsTempate);
                    if (diff < minimumDiff) {
                        minimumDiff = diff;
                        minNum = j;
                        angleOfMin =  NL.numbers[j].downwardAngles[k];
                    }
                }
            }
            if (minimumDiff < 50 && minNum != -1) {
                numbers[i].bIsANumber = true;
                numbers[i].whichNum = minNum;
                numbers[i].downwardAngle = angleOfMin;
                numbers[i].myDownAngle =  numbers[i].angle - (PI/2.0f -   numbers[i].downwardAngle);
            } else {
                numbers[i].bPossibleNumber = false;
                numbers[i].bIsANumber = false;
                numbers[i].myDownAngle = 0;

            }
        }
    }

    for (int i = 0; i < ofCvF.nBlobs; i++) {
        if (numbers[i].bPossibleNumber == true) {
            numbers[i].ptToCheckIfBlobIsNear.set(numbers[i].midPt.x + 25 * cos(numbers[i].angle - (PI/2.0f - numbers[i].downwardAngle)), numbers[i].midPt.y + 25 * sin( numbers[i].angle -  (PI/2.0f - numbers[i].downwardAngle)));


            float minDis = 1000000;
            for (int j = 0; j < ofCvF.nBlobs; j++) {
                if ((i != j) && numbers[j].bPossibleNumber == false) {
                    ofxVec2f temp = numbers[i].ptToCheckIfBlobIsNear;
                    temp -= numbers[j].midPt;

                    float dist = sqrt (temp.x * temp.x + temp.y * temp.y);
                    if (minDis > dist) minDis = dist;
                }
            }

            if (minDis < 15){
                numbers[i].bAmIUpright = true;
            } else {
                numbers[i].bPossibleNumber = false;
                numbers[i].bIsANumber = false;
            }
            //printf("checking for uprightness == %f \n", minDis);
        }
    }



    //cvCardBlob          numbers[128];
    //int                         nNumbers;


}

//--------------------------------------------------------------
void cvManager::draw() {

    if (drawMode == 0) {
        glColor3f(1,1,1);
        glScalef(0.7,0.7,1);
        cvInputColor.draw(0,0, 320,240);
        CS.cvInputNonCard.draw(w/2,0,320,240);
        CS.cvInputDilatedHeavily.draw(w*2/2,0,320,240);
        CS.cvInputDiff.draw(0,h);
        glTranslatef(1024,480,0);
        NL.draw();
    } else if (drawMode == 1) {
        cvInputColor.draw(0,0, 640,480);
        for (int i = 0; i < nNumbers; i++) {
            if ( numbers[i].bPossibleNumber) {
                ofSetColor(0xffffff);
                ofNoFill();
                numbers[i].draw();
            } else {
                ofSetColor(0x333333);
                ofNoFill();
                numbers[i].draw();
            }

            /*ofSetColor(0xdddddd);
            string output = "";
            output += ofToString(numbers[i].aspectRatio, 2);
            ofDrawBitmapString(output, numbers[i].contourPts[0].x + 20, numbers[i].contourPts[0].y);
            */
        }

    } else {


    }

    ofSetColor(0xffffff);
    string output;
    output = "info on tracking\n\n";
    output += "nonCardTheshold (a/s): " + ofToString(CS.nonCardTheshold) + "\n";
    output += "bgSubTheshold (z/x): " + ofToString(CS.bgSubTheshold) + "\n";
    output += "minArea (d/c): " + ofToString(minArea, 2) + "\n";
    output += "maxArea (f/v) : " + ofToString(maxArea, 2) + "\n";
    output += "minRatio (g/b) : " + ofToString(minRatio, 2) + "\n";
    ofDrawBitmapString(output, 500,500);

}

//--------------------------------------------------------------
void cvManager::learnNumbers(int numberset) {

    ofPoint dst[4];
    dst[0].x = 0;
    dst[0].y = 0;
    dst[1].x = 64;
    dst[1].y = 0;
    dst[2].x = 64;
    dst[2].y = 64;
    dst[3].x = 0;
    dst[3].y = 64;
    for (int i = 0; i < nNumbers; i++) {
        if (numbers[i].bPossibleNumber == true) {
            numberTemplate = new ofxCvGrayscaleImage();
            numberTemplate->allocate(64, 64);
            numberTemplate->warpIntoMe(cvInputGray, numbers[i].quadPts, dst);
            numberTemplate->contrastStretch();
            NL.learnThisNumber(numberset, numberTemplate);
        } else {
            //printf("skipping \n");
        }
    }
}


//--------------------------------------------------------------
void cvManager::saveSettings() {
    settings.loadFile("someFileThatDoesntExist.xml");
    settings.setVerbose(true);
    settings.setValue("param:cv:nonCardTheshold", CS.nonCardTheshold);
    settings.setValue("param:cv:bgSubTheshold", CS.bgSubTheshold);
    settings.setValue("param:cv:minArea", minArea);
    settings.setValue("param:cv:maxArea", maxArea);
    settings.saveFile("settings.xml");
}

//--------------------------------------------------------------
void cvManager::loadSettings() {
    settings.loadFile("settings.xml");
    CS.nonCardTheshold = settings.getValue("param:cv:nonCardTheshold", CS.nonCardTheshold);
    CS.bgSubTheshold = settings.getValue("param:cv:bgSubTheshold", CS.bgSubTheshold);
    minArea = settings.getValue("param:cv:minArea", minArea );
    maxArea = settings.getValue("param:cv:maxArea", maxArea);
    settings.clear();
}


